from unittest import TestCase, mock
import pytest

import cnvrg_experiment_chart


__author__ = "Craig Smith"
__copyright__ = "Craig Smith"
__license__ = "MIT"


class TestInit(TestCase):
    @mock.patch("cnvrg_experiment_chart.chart.Experiment.__init__")
    def test_init_experiment_chart_good_wo_obj(self, mock_exp_init):
        mock_exp_init.return_value = None
        test_chart = cnvrg_experiment_chart.ExperimentChart(
            key="test_key", chart_type="line"
        )
        self.assertEqual(test_chart.key, "test_key")
        self.assertEqual(test_chart.chart_type, "none")
        self.assertIsInstance(
            test_chart.experiment, cnvrg_experiment_chart.chart.Experiment
        )
        self.assertIsInstance(test_chart.chart, cnvrg_experiment_chart.chart.Chart)

    @mock.patch("cnvrg_experiment_chart.chart.Experiment.__init__")
    def test_init_experiment_chart_good_w_obj(self, mock_exp_init):
        mock_exp_init.return_value = None
        test_exp = cnvrg_experiment_chart.chart.Experiment()
        test_chart = cnvrg_experiment_chart.ExperimentChart(
            key="test_key", chart_type="line", experiment=test_exp
        )
        self.assertIsInstance(
            test_chart.experiment, cnvrg_experiment_chart.chart.Experiment
        )

    @mock.patch("cnvrg_experiment_chart.chart.Experiment.__init__")
    def test_init_experiment_chart_w_bad_chart_type(self, mock_exp_init):
        mock_exp_init.return_value = None
        with pytest.raises(Exception) as e_info:
            test_chart = cnvrg_experiment_chart.ExperimentChart(
                key="test_key", chart_type="bad"
            )
            del test_chart
            assert e_info is not None

    @mock.patch("cnvrg_experiment_chart.chart.Experiment.__init__")
    def test_init_experiment_chart_good_nonline(self, mock_exp_init):
        mock_exp_init.return_value = None
        test_chart = cnvrg_experiment_chart.ExperimentChart(
            key="test_key", chart_type="bar"
        )
        self.assertEqual(test_chart.key, "test_key")
        self.assertEqual(test_chart.chart_type, "bar")
        self.assertIsInstance(
            test_chart.experiment, cnvrg_experiment_chart.chart.Experiment
        )
        self.assertIsInstance(test_chart.chart, cnvrg_experiment_chart.chart.Chart)


class TestCreateChart(TestCase):
    @mock.patch("cnvrg_experiment_chart.chart.Experiment.__init__")
    def setUp(self, mock_exp_init):
        mock_exp_init.return_value = None
        self.test_exp_chart = cnvrg_experiment_chart.ExperimentChart(
            key="test_key", chart_type="line"
        )
        self.addCleanup(mock.patch.stopall)

    @mock.patch("cnvrg_experiment_chart.chart.Experiment.log_metric")
    def test_create_chart(self, mock_log_metric):
        mock_log_metric.return_value = None
        test_return_obj = self.test_exp_chart.create_chart()
        self.assertIsInstance(test_return_obj, cnvrg_experiment_chart.ExperimentChart)


class TestAddSeries(TestCase):
    @mock.patch("cnvrg_experiment_chart.chart.Experiment.__init__")
    def setUp(self, mock_exp_init):
        mock_exp_init.return_value = None
        self.test_exp_chart = cnvrg_experiment_chart.ExperimentChart(
            key="test_key", chart_type="line"
        )
        self.addCleanup(mock.patch.stopall)

    def test_add_series_no_data(self):
        test_obj = self.test_exp_chart.add_series("test_series")
        self.assertIsInstance(test_obj, cnvrg_experiment_chart.ExperimentChart)

    def test_add_series_data(self):
        test_obj = self.test_exp_chart.add_series(series_name="test_series", data=[1])
        self.assertIsInstance(test_obj, cnvrg_experiment_chart.ExperimentChart)


class TestAddMetric(TestCase):
    @mock.patch("cnvrg_experiment_chart.chart.Experiment.__init__")
    def setUp(self, mock_exp_init):
        mock_exp_init.return_value = None
        self.test_exp_chart = cnvrg_experiment_chart.ExperimentChart(
            key="test_key", chart_type="line"
        )
        self.addCleanup(mock.patch.stopall)

    @mock.patch("cnvrg_experiment_chart.chart.Experiment.update_chart")
    def test_add_metric(self, mock_update_chart):
        mock_update_chart.return_value = None
        test_obj = self.test_exp_chart.add_metric([1], "test_series")
        self.assertIsInstance(test_obj, cnvrg_experiment_chart.ExperimentChart)



"""Module to supliment the handling of charts in the cnvrgv2 sdk"""
from cnvrgv2.utils.chart_utils import Chart
from cnvrgv2.modules.workflows.experiment.experiment import Experiment


class ExperimentChart:
    """
    A class used to manage an experiment's chart

    Attributes
    ----------
    experiment : cnvrgv2.Experiment
        A cnvrg Experiment object. If you do not pass in, we assume you want us
        to create this on your behalf.
    key : str
        The title and reference name used for the chart
    chart_type : str
        The type of chart to generate
    chart : LineChart
        The cnvrg Chart object generated after instantiating this object
    Examples
    --------
    >>> chart = ExperimentChart(key="line_chart", chart_type="line")
    >>> chart.add_series(series_name="index")
    >>> chart.create_chart()
    >>> chart.add_metric(series_name="index", data=[1,2,3])
    """

    def __init__(self, **kwargs):
        """
        Parameters
        ----------
        key : str
            The title and reference name used for the chart
        chart_type : str
            The type of chart to generate. Choices are: line todo: bar, heatmap
            and scatter
        group : string
            Not sure
        step : int
            Number of steps?
        x_ticks : list
            List of strings to group together bar charts?
        stops : int
            Not sure
        max_val : int
            Maximum boundary for the Y axis
        min_val : int
            Minimum boundary for the Y axis
        colors : string
        Not sure, not valid with chart_type: line
        experiment: cnvrgv2.Experiment
            A cnvrg Experiment object. If you do not pass in, we assume you
            want us to create this on your behalf.
        """
        valid_types = ["line", "bar", "scatter", "heatmap"]
        if kwargs["chart_type"] not in valid_types:
            raise AttributeError(
                f"""Chart type must be one of the following strings:
                {valid_types}"""
            )
        self.experiment = kwargs.get("experiment", None)
        self.key = kwargs["key"]
        self.chart_type = kwargs["chart_type"]
        if self.chart_type == "line":
            self.chart_type = "none"
        self.chart = Chart(
            key=self.key,
            chart_type=self.chart_type,
            group=kwargs.get("group", None),
            step=kwargs.get("step", None),
            x_ticks=kwargs.get("x_ticks", None),
            stops=kwargs.get("stops", None),
            max_val=kwargs.get("max_val", None),
            min_val=kwargs.get("min_val", None),
            colors=kwargs.get("colors", None),
        )
        if not isinstance(self.experiment, Experiment):
            self.experiment = Experiment()

    def create_chart(self):
        """
        This method will add your chart to the experiment (aka. adds the chart
        to the experiments UI)

        Returns
        -------
        self : ExperimentChart
            Returns the updated ExperimentChart object allowing for method
            chaining
        """
        self.chart = self.experiment.log_metric(self.chart)
        return self

    def add_series(self, series_name, data=None):
        """
        This method adds a data series to the chart as well as the initial data
        point

        Parameters
        ----------
        data : list, default=None
            This is a list of one or more data points that will be added to the
            data series. If not included this will create an empty series.
        series_name : str
            This is the name you would like to give the data series.

        Returns
        -------
        self : ExperimentChart
            Returns the updated ExperimentChart object allowing for method
            chaining
        """
        if data is None:
            data = []
        self.chart.add_series(data, series_name)
        return self

    def add_metric(self, data, series_name):
        """
        This method appends a data point to an existing data series (appears in
        UI in real time)

        Parameters
        ----------
        data : list
            This is a list of one or more data points that will be appended to
            the currently list of datapoints
        series_name : str
            This is the name of the series to add the datapoints to

        Returns
        -------
        self : ExperimentChart
            Returns the updated ExperimentChart object allowing for method
            chaining
        """
        self.chart = self.experiment.update_chart(self.key, data, series_name)
        return self
